package org.xbib.io.pool.jdbc;

import org.junit.jupiter.api.Test;
import org.xbib.jdbc.connection.pool.util.DriverDataSource;

import java.util.Arrays;
import java.util.List;
import java.util.Properties;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.junit.jupiter.api.Assertions.fail;

public class DriverDataSourceTest {

   @Test
   public void testJdbcUrlLogging() {
      List<String> urls = Arrays.asList(
         "jdbc:invalid://host/d_dlq?user=USER&password=SECRET",
         "jdbc:invalid://host/d_dlq?user=USER&truststorePassword=SECRET",
         "jdbc:invalid://host/d_dlq?a=b&password=SECRET&user=USER",
         "jdbc:invalid://host/d_dlq?a=b&sslpassword=SECRET&user=USER",
         "jdbc:invalid://host/d_dlq?a=b&sslpassword=SECRET&password=SECRET&user=USER",
         "jdbc:invalid://host/d_dlq?truststorePassword=SECRET;user=USER&password=SECRET#extra",
         "jdbc:invalid://host/d_dlq?sslpassword=SECRET&password=SECRET&trustPassword=SECRET&user=USER",
         "jdbc:invalid://host/d_dlq?password=SECRET#user=USER;extra"
      );
      for (String url : urls) {
         testExceptionMessage(url);
      }
   }

   private void testExceptionMessage(String jdbcUrl) {
      try {
         new DriverDataSource(jdbcUrl, null, new Properties(), null, null);
         fail();
      } catch (RuntimeException e) {
         String msg = e.getMessage();
         assertTrue(msg.contains("jdbc:invalid://host/d_dlq"));
         assertTrue(msg.contains("user=USER"));
         assertFalse(msg.contains("SECRET"));
      }
   }
}
