package org.xbib.jdbc.mariadb;

import org.xbib.jdbc.query.Flavor;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Locale;

public class MariaDB implements Flavor {

    public MariaDB() {
    }

    @Override
    public String getName() {
        return "mariadb";
    }

    @Override
    public boolean supports(String url) {
        return url.startsWith("jdbc:mariadb:");
    }

    @Override
    public String driverClass() {
        return "org.mariadb.jdbc.Driver";
    }

    @Override
    public String normalizeTableName(String tableName) {
        if (tableName == null) {
            return tableName;
        }
        if (tableName.length() > 2) {
            if (tableName.startsWith("\"") && tableName.endsWith("\"")) {
                return tableName.substring(1, tableName.length() - 1);
            }
        }
        return tableName.toLowerCase(Locale.ROOT);
    }

    @Override
    public String typeInteger() {
        return "integer";
    }

    @Override
    public String typeBoolean() {
        return "char(1)";
    }

    @Override
    public String typeLong() {
        return "bigint";
    }

    @Override
    public String typeFloat() {
        return "double";
    }

    @Override
    public void setFloat(PreparedStatement preparedStatement, int i, Float floatValue) throws SQLException {
        preparedStatement.setDouble(i, floatValue);
    }

    @Override
    public String typeDouble() {
        return "double";
    }

    @Override
    public void setDouble(PreparedStatement preparedStatement, int i, Double doubleValue) throws SQLException {
        preparedStatement.setDouble(i, doubleValue);
    }

    @Override
    public String typeBigDecimal(int size, int precision) {
        return "numeric(" + size + "," + precision + ")";
    }

    @Override
    public String typeStringVar(int length) {
        if (length == 16777215) {
            return "mediumtext";
        }
        return "varchar(" + length + ")";
    }

    @Override
    public String typeStringFixed(int length) {
        return "char(" + length + ")";
    }

    @Override
    public String typeClob() {
        return "mediumtext"; // mediumtext = varchar(16777215), longtext = varchar(0)
    }

    @Override
    public String typeBlob() {
        return "mediumblob";
    }

    @Override
    public String typeLocalDateTime() {
        return "datetime(3)";
    }

    @Override
    public String columnTypeLocalDateTime() {
        return "DATETIME";
    }

    @Override
    public String typeLocalDate() {
        return "date";
    }

    @Override
    public boolean useStringForClob() {
        return true;
    }

    @Override
    public boolean useBytesForBlob() {
        return true;
    }

    @Override
    public String sequenceNextVal(String sequenceName) {
        return "next value for " + sequenceName + "";
    }

    @Override
    public String sequenceSelectNextVal(String sequenceName) {
        return "select " + sequenceNextVal(sequenceName) + fromAny();
    }

    @Override
    public String sequenceDrop(String dbtestSeq) {
        return "drop sequence if exists " + dbtestSeq;
    }

    @Override
    public String tableDrop(String table) {
        return "drop table if exists " + table;
    }

    @Override
    public String sequenceOrderClause(boolean order) {
        return "";
    }

    @Override
    public String sequenceCycleClause(boolean cycle) {
        return "";
    }

    @Override
    public String fromAny() {
        return "";
    }

    @Override
    public boolean supportsInsertReturning() {
        return false;
    }

    @Override
    public String dbTimeMillis() {
        return "localtimestamp(3)";
    }

    @Override
    public String sequenceCacheClause(int nbrValuesToCache) {
        return " cache " + nbrValuesToCache;
    }

    @Override
    public String sequenceOptions() {
        return "";
    }

    @Override
    public boolean isAutoCommitOnly() {
        return false;
    }

    @Override
    public String queueLength(String table) {
        return "select count(*) from " + table
                + " where channel = ? and state is NULL";
    }

    @Override
    public String writeNextIntoQueue(String table) {
        return "insert into " + table
                + " (channel, data, state, delay, modified) "
                + "values (?, ?, null, 0, ?)";
    }

    @Override
    public String readNextFromQueue(String table) {
        return "select id, data from " + table +
                " where channel = ? and state is NULL and pushed_at <= (CAST (? as TIMESTAMP) - (INTERVAL '1 minute' * delay))" +
                " order by id asc limit 1 for update skip lock";
    }

    @Override
    public String succeedInQueue(String table) {
        return "update" + table + " set state = 'OK', modified = " + dbTimeMillis() + " where id = ?";
    }

    @Override
    public String failInQueue(String table) {
        return "update" + table + " set state = 'FAIL', modified = " + dbTimeMillis() + " where id = ?";
    }
}
