package org.xbib.jdbc.oracle;

import oracle.jdbc.OraclePreparedStatement;
import org.xbib.jdbc.connection.pool.ProxyPreparedStatement;
import org.xbib.jdbc.query.Flavor;

import java.sql.PreparedStatement;
import java.sql.SQLException;

public class Oracle implements Flavor {

    public Oracle() {
    }

    @Override
    public String getName() {
        return "oracle";
    }

    @Override
    public boolean supports(String url) {
        return url.startsWith("jdbc:oracle:");
    }

    @Override
    public String driverClass() {
        return "oracle.jdbc.OracleDriver";
    }

    @Override
    public String normalizeTableName(String tableName) {
        if (tableName == null) {
            return tableName;
        }
        if (tableName.length() > 2) {
            if (tableName.startsWith("\"") && tableName.endsWith("\"")) {
                return tableName.substring(1, tableName.length() - 1);
            }
        }
        return tableName.toUpperCase();
    }

    @Override
    public String typeFloat() {
        return "binary_float";
    }

    @Override
    public void setFloat(PreparedStatement preparedStatement, int i, Float floatValue) throws SQLException {
        if (preparedStatement instanceof ProxyPreparedStatement) {
            ProxyPreparedStatement proxyPreparedStatement = (ProxyPreparedStatement) preparedStatement;
            ((OraclePreparedStatement) proxyPreparedStatement.getDelegate()).setBinaryFloat(i, floatValue);
        } else {
            ((OraclePreparedStatement) preparedStatement).setBinaryFloat(i, floatValue);
        }
    }

    @Override
    public String typeDouble() {
        return "binary_double";
    }

    @Override
    public void setDouble(PreparedStatement preparedStatement, int i, Double doubleValue) throws SQLException {
        if (preparedStatement instanceof ProxyPreparedStatement) {
            ProxyPreparedStatement proxyPreparedStatement = (ProxyPreparedStatement) preparedStatement;
            ((OraclePreparedStatement) proxyPreparedStatement.getDelegate()).setBinaryDouble(i, doubleValue);
        } else {
            ((OraclePreparedStatement) preparedStatement).setBinaryDouble(i, doubleValue);
        }
    }

    @Override
    public String typeBigDecimal(int size, int precision) {
        return "numeric(" + size + "," + precision + ")";
    }

    @Override
    public String typeInteger() {
        return "numeric(10)";
    }

    @Override
    public String typeBoolean() {
        return "char(1 char)";
    }

    @Override
    public String typeLong() {
        return "numeric(19)";
    }

    @Override
    public String typeLocalDateTime() {
        return "timestamp(3)";
    }

    @Override
    public String columnTypeLocalDateTime() {
        return "TIMESTAMP";
    }

    @Override
    public String typeLocalDate() {
        return "date";
    }

    @Override
    public boolean useStringForClob() {
        return false;
    }

    @Override
    public boolean useBytesForBlob() {
        return false;
    }

    @Override
    public String sequenceNextVal(String sequenceName) {
        return sequenceName + ".nextval";
    }

    @Override
    public String sequenceSelectNextVal(String sequenceName) {
        return "select " + sequenceName + ".nextval from dual";
    }

    @Override
    public String sequenceDrop(String sequenceName) {
        return "drop sequence " + sequenceName;
    }

    @Override
    public String tableDrop(String table) {
        return "drop table " + table;
    }

    @Override
    public String typeStringVar(int length) {
        return "varchar2(" + length + " char)";
    }

    @Override
    public String typeStringFixed(int length) {
        return "char(" + length + " char)";
    }

    @Override
    public String typeClob() {
        return "clob";
    }

    @Override
    public String typeBlob() {
        return "blob";
    }

    @Override
    public String sequenceOrderClause(boolean order) {
        return order ? " order" : " noorder";
    }

    @Override
    public String sequenceCycleClause(boolean cycle) {
        return cycle ? " cycle" : " nocycle";
    }

    @Override
    public boolean supportsInsertReturning() {
        return true;
    }

    @Override
    public String dbTimeMillis() {
        return "systimestamp(3)";
    }

    @Override
    public String sequenceCacheClause(int nbrValuesToCache) {
        if (nbrValuesToCache < 2) {
            return " nocache";
        }
        return " cache " + nbrValuesToCache;
    }

    @Override
    public String fromAny() {
        return " from dual";
    }

    @Override
    public String sequenceOptions() {
        return "";
    }

    @Override
    public boolean isAutoCommitOnly() {
        return false;
    }

    @Override
    public String queueLength(String table) {
        return "select count(*) from " + table
                + " where channel = ? and state is NULL";
    }

    @Override
    public String writeNextIntoQueue(String table) {
        return "insert into " + table
                + " (channel, data, state, delay, modified) "
                + "values (?, ?, null, 0, ?)";
    }

    @Override
    public String readNextFromQueue(String table) {
        return "select id, data from " + table +
                " where channel = ? and state is NULL and modified <= (CAST (? as TIMESTAMP) - (INTERVAL '1 minute' * delay))" +
                " order by id asc limit 1 for update skip lock";
    }

    @Override
    public String succeedInQueue(String table) {
        return "update" + table + " set state = 'OK', modified = " + dbTimeMillis() + " where id = ?";
    }

    @Override
    public String failInQueue(String table) {
        return "update" + table + " set state = 'FAIL', modified = " + dbTimeMillis() + " where id = ?";
    }
}
