package org.xbib.jdbc.postgresql.test;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.testcontainers.containers.PostgreSQLContainer;
import org.xbib.jdbc.query.Config;
import org.xbib.jdbc.query.ConfigSupplier;
import org.xbib.jdbc.query.DatabaseProvider;
import org.xbib.jdbc.query.OptionsOverride;
import org.xbib.jdbc.query.Schema;
import org.xbib.jdbc.test.CommonTest;

/**
 * Exercise Database functionality with a real PostgreSQL database.
 */
public class PostgresqlTest extends CommonTest {

    static PostgreSQLContainer<?> postgreSQLContainer;

    static {
        // postgresql 9.6.12
        postgreSQLContainer = new PostgreSQLContainer<>("postgres")
                .withDatabaseName("testDB")
                .withUsername("testUser")
                .withPassword("testPassword");
    }

    @BeforeAll
    public static void startContainer() {
        postgreSQLContainer.start();
    }

    @AfterAll
    public static void stopContainer() {
        postgreSQLContainer.stop();
    }

    @Override
    protected DatabaseProvider createDatabaseProvider(OptionsOverride options) throws Exception {
        Config config = ConfigSupplier.of()
                .property("database.url", postgreSQLContainer.getJdbcUrl())
                .property("database.user", "testUser")
                .property("database.password", "testPassword")
                .get();
        return DatabaseProvider.builder(config)
                .withSqlParameterLogging()
                .withSqlInExceptionMessages()
                .withOptions(options)
                .build();
    }

    /**
     * PostgreSQL seems to have different behavior in that is does not convert
     * column names to uppercase (it actually converts them to lowercase).
     * I haven't figured out how to smooth over this difference, since all databases
     * seem to respect the provided case when it is inside quotes, but don't provide
     * a way to tell whether a particular parameter was quoted.
     */
    @Override
    @Test
    public void metadataColumnNames() {
        db.dropTableQuietly("dbtest");
        new Schema().addTable("dbtest").addColumn("pk").primaryKey().schema().execute(db);
        db.toSelect("select Pk, Pk as Foo, Pk as \"Foo\" from dbtest")
                .query(rs -> {
                    Assertions.assertArrayEquals(new String[]{"pk", "foo", "Foo"}, rs.getColumnLabels());
                    return null;
                });
    }
}
