package org.xbib.jdbc.query;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ServiceLoader;

public interface Flavor {

    String getName();

    boolean supports(String url);

    String driverClass();

    String normalizeTableName(String tableName);

    String typeInteger();

    String typeBoolean();

    String typeLong();

    String typeFloat();

    void setFloat(PreparedStatement preparedStatement, int i, Float floatValue) throws SQLException;

    String typeDouble();

    void setDouble(PreparedStatement preparedStatement, int i, Double doubleValue) throws SQLException;

    String typeBigDecimal(int size, int precision);

    String typeStringVar(int length);

    String typeStringFixed(int length);

    String typeClob();

    String typeBlob();

    String typeLocalDateTime();

    String columnTypeLocalDateTime();

    String typeLocalDate();

    boolean useStringForClob();

    boolean useBytesForBlob();

    String sequenceNextVal(String sequenceName);

    String sequenceSelectNextVal(String sequenceName);

    String sequenceDrop(String sequenceName);

    String tableDrop(String table);

    boolean supportsInsertReturning();

    String dbTimeMillis();

    String sequenceCacheClause(int nbrValuesToCache);

    String sequenceOrderClause(boolean order);

    String sequenceCycleClause(boolean cycle);

    /**
     * Indicate what should follow a constant select statement. For example, "select 1"
     * works on some databases, while Oracle requires "select 1 from dual". For Oracle
     * this function should return " from dual" (including the leading space).
     */
    String fromAny();

    String sequenceOptions();

    /**
     * In JDBC, all drivers are set to autocommit by default.
     * @return Returns true if we let the JDBC driver exclusively decide about commit(). If false, it allows explicit commit() calls.
     */
    boolean isAutoCommitOnly();

    String queueLength(String table);

    String writeNextIntoQueue(String table);

    String readNextFromQueue(String table);

    String succeedInQueue(String table);

    String failInQueue(String table);

    static Flavor fromJdbcUrl(String url) {
        if (url == null) {
            throw new DatabaseException("url must not be null");
        }
        ServiceLoader<Flavor> serviceLoader = ServiceLoader.load(Flavor.class);
        for (Flavor flavor : serviceLoader) {
            if (flavor.supports(url)) {
                return flavor;
            }
        }
        throw new DatabaseException("Cannot determine database flavor from url");
    }

    static Flavor valueOf(String name) {
        if (name == null) {
            throw new DatabaseException("name must not be null");
        }
        ServiceLoader<Flavor> serviceLoader = ServiceLoader.load(Flavor.class);
        for (Flavor flavor : serviceLoader) {
            if (flavor.getName().equals(name)) {
                return flavor;
            }
        }
        throw new DatabaseException("Cannot determine database flavor from name: " + name);
    }
}
