package org.xbib.jdbc.query;

import java.time.LocalDateTime;

/**
 * Control various optional behavior for the database interactions.
 */
public class OptionsDefault implements Options {

    private final Flavor flavor;

    public OptionsDefault(Flavor flavor) {
        this.flavor = flavor;
    }

    @Override
    public int fetchSize() {
        return 10;
    }

    @Override
    public int timeoutSeconds() {
        return 15;
    }

    @Override
    public boolean allowTransactionControl() {
        return false;
    }

    @Override
    public boolean ignoreTransactionControl() {
        return false;
    }

    @Override
    public boolean allowConnectionAccess() {
        return false;
    }

    @Override
    public boolean isLogParameters() {
        return false;
    }

    @Override
    public boolean isDetailedExceptions() {
        return false;
    }

    @Override
    public String generateErrorCode() {
        return LocalDateTime.now() + "-" + Math.round(Math.random() * 1000000);
    }

    @Override
    public boolean useBytesForBlob() {
        return flavor().useBytesForBlob();
    }

    @Override
    public boolean useStringForClob() {
        return flavor().useStringForClob();
    }

    @Override
    public Flavor flavor() {
        return flavor;
    }

    @Override
    public boolean useLocalDateTimeOnly() {
        return false;
    }

    @Override
    public int maxStringLengthParam() {
        return 4000;
    }
}
